#!/usr/bin/python3.7
# ads1256.py
#
# Copyright (c) 2020 Spring City Solutions LLC and other contributors
# Licensed under the Apache License, Version 2.0
#
# Need to have the rpi.gpio and spidev python libraries installed
#
# https://www.waveshare.com/High-Precision-AD-DA-Board.htm
# https://www.waveshare.com/wiki/High-Precision_AD/DA_Board
# https://github.com/waveshare/High-Precision-AD-DA-Board
# https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
# https://npmjs.org/package/node-red-contrib-waveshare-shield-adda-11010
# https://flows.nodered.org/node/node-red-contrib-waveshare-shield-adda-11010
# https://nodered.org/
# Links for ADS1256 chip:
# https://www.ti.com/product/ADS1256
# https://www.ti.com/lit/gpn/ads1256
# Link for LM285 chip, connected to the ADS1256:
# https://www.ti.com/product/LM285-2.5-N
# https://www.ti.com/lit/gpn/lm285-2.5-n
# Link for DAC8532 chip:
# https://www.ti.com/product/DAC8532
# https://www.ti.com/lit/gpn/dac8532
#
# Usage:
#
# ads1256.py analog "channel number 1 to 8"
# returns 0 for 0 volts and decimal 0xffffff for 5 volts
#
# ads1256.py digitalIn "D0 to D3"
# returns 0 or 1 as per input to the specified pin
#
# ads1256.py digitalOut "D0 to D3" "0 or 1"
#

import RPi.GPIO as GPIO
import spidev
import sys

cs = 22
drdy = 17
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
GPIO.setup(cs, GPIO.OUT)
GPIO.output(cs, 1)
GPIO.setup(drdy, GPIO.IN)

spi = spidev.SpiDev()
spi.open(0, 0)
spi.max_speed_hz = 20000
spi.mode = 0b01


def ReadReg(register):
    GPIO.output(cs, 0)
    firstbyte = 0x10 + register
    secondbyte = 0x01
    spi.writebytes([firstbyte, secondbyte])
    data = spi.readbytes(1)
    GPIO.output(cs, 1)
    return data[0]


def PrintRegisters():
    print('0x00 0x' + format(ReadReg(0x00), '02x') + ' Status')
    print('0x01 0x' + format(ReadReg(0x01), '02x') + ' MUX')
    print('0x02 0x' + format(ReadReg(0x02), '02x') + ' ADCON')
    print('0x03 0x' + format(ReadReg(0x03), '02x') + ' DRATE')
    print('0x04 0x' + format(ReadReg(0x04), '02x') + ' IO')
    print('0x05 0x' + format(ReadReg(0x05), '02x') + ' OFC0')
    print('0x06 0x' + format(ReadReg(0x06), '02x') + ' OFC1')
    print('0x07 0x' + format(ReadReg(0x07), '02x') + ' OFC2')
    print('0x08 0x' + format(ReadReg(0x08), '02x') + ' FSC0')
    print('0x09 0x' + format(ReadReg(0x09), '02x') + ' FSC1')
    print('0x0A 0x' + format(ReadReg(0x0A), '02x') + ' FSC2')
    print()


def WriteReg(register, data):
    GPIO.output(cs, 0)
    firstbyte = 0x50 + register
    secondbyte = 0x01
    spi.writebytes([firstbyte, secondbyte, data])
    GPIO.output(cs, 1)


def WriteCmd(command):
    GPIO.output(cs, 0)
    spi.writebytes([command])
    GPIO.output(cs, 1)


def SELFCAL():
    WriteCmd(0xf0)
    while True:
        if (GPIO.input(drdy) == 0):
            break


def RDATA():
    while True:
        if (GPIO.input(drdy) == 0):
            break
    GPIO.output(cs, 0)
    spi.writebytes([0x01])
    buf = spi.readbytes(3)
    GPIO.output(cs, 1)
    read = (buf[0] << 16) & 0xff0000
    read |= (buf[1] << 8) & 0xff00
    read |= (buf[2]) & 0xff
    if (read & 0x800000):
        read &= 0xF000000
    return read


# Shut off pin D0 clock out, shut off SDCS, set gain to x1
WriteReg(0x02, 0x00)

if (sys.argv[1] == 'analog'):
    # Data Rate to highest resolution 2.5 SPS
    WriteReg(0x03, 0x03)
    # PrintRegisters()
    SELFCAL()
    # PrintRegisters()
    # Set the channel
    channelbyte = ((int(sys.argv[2]) - 1) << 4) | (1 << 3)
    WriteReg(0x01, channelbyte)
    # Sync and wakeup the converter
    WriteCmd(0xfc)
    WriteCmd(0xff)
    # Read data
    print(RDATA())

if (sys.argv[1] == 'digitalIn'):
    directionbitmask = 0x10
    inputbitmask = 0x01
    if (sys.argv[2] == 'D0'):
        directionbitmask = 0x10
        inputbitmask = 0x01
    if (sys.argv[2] == 'D1'):
        directionbitmask = 0x20
        inputbitmask = 0x02
    if (sys.argv[2] == 'D2'):
        directionbitmask = 0x40
        inputbitmask = 0x04
    if (sys.argv[2] == 'D3'):
        directionbitmask = 0x80
        inputbitmask = 0x08
    oldregister = ReadReg(0x04)
    WriteReg(0x04, oldregister | directionbitmask)
    if ((ReadReg(0x04) & inputbitmask) > 0):
        print('1')
    else:
        print('0')

if (sys.argv[1] == 'digitalOut'):
    directionbitmask = 0x10
    outputbitmask = 0x01
    if (sys.argv[2] == 'D0'):
        directionbitmask = 0x10
        outputbitmask = 0x01
    if (sys.argv[2] == 'D1'):
        directionbitmask = 0x20
        outputbitmask = 0x02
    if (sys.argv[2] == 'D2'):
        directionbitmask = 0x40
        outputbitmask = 0x04
    if (sys.argv[2] == 'D3'):
        directionbitmask = 0x80
        outputbitmask = 0x08
    oldregister = ReadReg(0x04)
    if (sys.argv[3] == '0'):
        WriteReg(0x04, oldregister & ~directionbitmask & ~outputbitmask)
    if (sys.argv[3] == '1'):
        WriteReg(0x04, oldregister & ~directionbitmask | outputbitmask)
